package MlFront_Exec

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module MlFront_Exec.BuildTaskFormSource

Sourceval mkdir : values_file:BuildCore.Io.file_object -> what:string -> Fmlib_parse.Position.range -> MlFront_Core.FilePath.t -> unit BuildCore.Alacarte_6_4_test.CSuspending.t
Sourceval rmdir : values_file:BuildCore.Io.file_object -> what:string -> Fmlib_parse.Position.range -> MlFront_Core.FilePath.t -> unit BuildCore.Alacarte_6_4_test.CSuspending.t

The helper function make_userformkind_task (ast,astfile) fetch creates a validated UserFormKind task from the source AST ast.

Never call this directly. Use BuildTaskFactory.add_form_tasks instead which correctly fetches a dependency on the source AST key so if the source AST is a dynamically created task it will change if the source AST changes.