package MlFront_Exec

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module SecPackageRegistry.DistributedModulesSource

The modules distributed by a distribution package.

Sourcetype t
Sourcetype is_distributed_response =
  1. | Distributed
  2. | NotDistributed of {
    1. available_majmin_versions : (int64 * int64) list;
    2. available_modules : MlFront_Core.StandardModuleId.t list;
    }
Sourceval from_distribution : MlFront_Thunk.ThunkDist.t -> t

from_distribution dist parses the build modules in the distribution dist.

Adds the major and minor components of all build module versions in dist to the set of distributed modules.

Sourceval is_distributed : MlFront_Core.StandardModuleId.t -> MlFront_Thunk.ThunkSemver64.t -> t -> is_distributed_response

is_distributed checks if the module module_id with the major and minor components of version semver is in the distributed modules dm.

Returns Distributed if the module is distributed.

Returns NotDistributed response if it is not, where response = { available_majmin_versions; available_modules } is both a list of (major, minor) pairs representing the distributed versions that are available for the module module_id and a list of all the available distributed modules.