package MlFront_Exec

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module MlFront_Exec.TracesSource

Code for traces.proto

Types

Sourcetype key_kind =
  1. | Keykind_unspecified
  2. | Keykind_user_form
  3. | Keykind_user_bundle
  4. | Keykind_user_asset
  5. | Keykind_checksum_valuesfile
  6. | Keykind_package_dist
Sourcetype standard_module_id = {
  1. vendor : string option;
  2. qualifier : string option;
  3. unit_ : string option;
  4. namespace_front : string list;
  5. namespace_tail : string option;
}
Sourcetype semver = {
  1. major : int64 option;
  2. minor : int64 option;
  3. patch : int64 option;
  4. prerelease : string list;
  5. build : string list;
}
Sourcetype module_version = {
  1. module_id : standard_module_id option;
  2. module_semver : semver option;
}
Sourcetype package_id = {
  1. vendor : string option;
  2. qualifier : string option;
  3. unit_ : string option;
  4. namespace : string list;
}
Sourcetype package_version = {
  1. package_id : package_id option;
  2. package_semver : semver option;
}
Sourcetype position = {
  1. line : int32 option;
  2. byte_bol : int32 option;
  3. byte_col : int32 option;
  4. correction : int32 option;
}
Sourcetype range = {
  1. start : position option;
  2. end_ : position option;
}
Sourcetype key = {
  1. keykind : key_kind option;
  2. debug_reference_file_sha256 : string option;
  3. checksum_sha256 : string option;
  4. values_canonical_id : string option;
  5. module_version : module_version option;
  6. package_version : package_version option;
  7. keykind_slot : string list;
  8. keykind_assetpath : string option;
  9. debug_reference_range : range option;
}
Sourcetype dependency = {
  1. depkey : key option;
  2. kvhash : string option;
}
Sourcetype value_type =
  1. | Valuetype_unspecified
  2. | Valuetype_form
  3. | Valuetype_object
  4. | Valuetype_constant
  5. | Valuetype_bundle
  6. | Valuetype_asset
  7. | Valuetype_valuesfile
  8. | Valuetype_values
  9. | Valuetype_distribution
Sourcetype checksum = {
  1. checksum_value : string option;
  2. checksum_range : range option;
}
Sourcetype value = {
  1. valuetype : value_type option;
  2. value_id : string option;
  3. value_sha256 : string option;
  4. valuesfile_canonical_id : string option;
  5. values_canonical_id : string option;
  6. values_file_sha256 : string option;
  7. values_file_local : string option;
  8. form_values_file_sha256 : string option;
  9. form_id : module_version option;
  10. form_values_canonical_id : string option;
  11. object_id : module_version option;
  12. object_range : range option;
  13. bundle_values_file_sha256 : string option;
  14. bundle_id : module_version option;
  15. bundle_range : range option;
  16. bundle_values_canonical_id : string option;
  17. bundle_values_file_local : string option;
  18. asset_values_file_sha256 : string option;
  19. asset_id : module_version option;
  20. asset_path : string option;
  21. asset_range : range option;
  22. asset_values_canonical_id : string option;
  23. asset_mirrors : string list;
  24. asset_checksum_sha256 : checksum option;
  25. asset_checksum_sha1 : checksum option;
  26. dist_id : package_version option;
  27. dist_json : string option;
}
Sourcetype constructive_trace = {
  1. key : key option;
  2. dependencies : dependency list;
  3. value : value option;
}

Basic values

Sourceval default_key_kind : unit -> key_kind

default_key_kind () is the default value for type key_kind

Sourceval default_standard_module_id : ?vendor:string option -> ?qualifier:string option -> ?unit_:string option -> ?namespace_front:string list -> ?namespace_tail:string option -> unit -> standard_module_id

default_standard_module_id () is the default value for type standard_module_id

Sourceval default_semver : ?major:int64 option -> ?minor:int64 option -> ?patch:int64 option -> ?prerelease:string list -> ?build:string list -> unit -> semver

default_semver () is the default value for type semver

Sourceval default_module_version : ?module_id:standard_module_id option -> ?module_semver:semver option -> unit -> module_version

default_module_version () is the default value for type module_version

Sourceval default_package_id : ?vendor:string option -> ?qualifier:string option -> ?unit_:string option -> ?namespace:string list -> unit -> package_id

default_package_id () is the default value for type package_id

Sourceval default_package_version : ?package_id:package_id option -> ?package_semver:semver option -> unit -> package_version

default_package_version () is the default value for type package_version

Sourceval default_position : ?line:int32 option -> ?byte_bol:int32 option -> ?byte_col:int32 option -> ?correction:int32 option -> unit -> position

default_position () is the default value for type position

Sourceval default_range : ?start:position option -> ?end_:position option -> unit -> range

default_range () is the default value for type range

Sourceval default_key : ?keykind:key_kind option -> ?debug_reference_file_sha256:string option -> ?checksum_sha256:string option -> ?values_canonical_id:string option -> ?module_version:module_version option -> ?package_version:package_version option -> ?keykind_slot:string list -> ?keykind_assetpath:string option -> ?debug_reference_range:range option -> unit -> key

default_key () is the default value for type key

Sourceval default_dependency : ?depkey:key option -> ?kvhash:string option -> unit -> dependency

default_dependency () is the default value for type dependency

Sourceval default_value_type : unit -> value_type

default_value_type () is the default value for type value_type

Sourceval default_checksum : ?checksum_value:string option -> ?checksum_range:range option -> unit -> checksum

default_checksum () is the default value for type checksum

Sourceval default_value : ?valuetype:value_type option -> ?value_id:string option -> ?value_sha256:string option -> ?valuesfile_canonical_id:string option -> ?values_canonical_id:string option -> ?values_file_sha256:string option -> ?values_file_local:string option -> ?form_values_file_sha256:string option -> ?form_id:module_version option -> ?form_values_canonical_id:string option -> ?object_id:module_version option -> ?object_range:range option -> ?bundle_values_file_sha256:string option -> ?bundle_id:module_version option -> ?bundle_range:range option -> ?bundle_values_canonical_id:string option -> ?bundle_values_file_local:string option -> ?asset_values_file_sha256:string option -> ?asset_id:module_version option -> ?asset_path:string option -> ?asset_range:range option -> ?asset_values_canonical_id:string option -> ?asset_mirrors:string list -> ?asset_checksum_sha256:checksum option -> ?asset_checksum_sha1:checksum option -> ?dist_id:package_version option -> ?dist_json:string option -> unit -> value

default_value () is the default value for type value

Sourceval default_constructive_trace : ?key:key option -> ?dependencies:dependency list -> ?value:value option -> unit -> constructive_trace

default_constructive_trace () is the default value for type constructive_trace

Protobuf Encoding

Sourceval encode_pb_key_kind : key_kind -> Pbrt.Encoder.t -> unit

encode_pb_key_kind v encoder encodes v with the given encoder

Sourceval encode_pb_standard_module_id : standard_module_id -> Pbrt.Encoder.t -> unit

encode_pb_standard_module_id v encoder encodes v with the given encoder

Sourceval encode_pb_semver : semver -> Pbrt.Encoder.t -> unit

encode_pb_semver v encoder encodes v with the given encoder

Sourceval encode_pb_module_version : module_version -> Pbrt.Encoder.t -> unit

encode_pb_module_version v encoder encodes v with the given encoder

Sourceval encode_pb_package_id : package_id -> Pbrt.Encoder.t -> unit

encode_pb_package_id v encoder encodes v with the given encoder

Sourceval encode_pb_package_version : package_version -> Pbrt.Encoder.t -> unit

encode_pb_package_version v encoder encodes v with the given encoder

Sourceval encode_pb_position : position -> Pbrt.Encoder.t -> unit

encode_pb_position v encoder encodes v with the given encoder

Sourceval encode_pb_range : range -> Pbrt.Encoder.t -> unit

encode_pb_range v encoder encodes v with the given encoder

Sourceval encode_pb_key : key -> Pbrt.Encoder.t -> unit

encode_pb_key v encoder encodes v with the given encoder

Sourceval encode_pb_dependency : dependency -> Pbrt.Encoder.t -> unit

encode_pb_dependency v encoder encodes v with the given encoder

Sourceval encode_pb_value_type : value_type -> Pbrt.Encoder.t -> unit

encode_pb_value_type v encoder encodes v with the given encoder

Sourceval encode_pb_checksum : checksum -> Pbrt.Encoder.t -> unit

encode_pb_checksum v encoder encodes v with the given encoder

Sourceval encode_pb_value : value -> Pbrt.Encoder.t -> unit

encode_pb_value v encoder encodes v with the given encoder

Sourceval encode_pb_constructive_trace : constructive_trace -> Pbrt.Encoder.t -> unit

encode_pb_constructive_trace v encoder encodes v with the given encoder

Protobuf Decoding

Sourceval decode_pb_key_kind : Pbrt.Decoder.t -> key_kind

decode_pb_key_kind decoder decodes a key_kind binary value from decoder

Sourceval decode_pb_standard_module_id : Pbrt.Decoder.t -> standard_module_id

decode_pb_standard_module_id decoder decodes a standard_module_id binary value from decoder

Sourceval decode_pb_semver : Pbrt.Decoder.t -> semver

decode_pb_semver decoder decodes a semver binary value from decoder

Sourceval decode_pb_module_version : Pbrt.Decoder.t -> module_version

decode_pb_module_version decoder decodes a module_version binary value from decoder

Sourceval decode_pb_package_id : Pbrt.Decoder.t -> package_id

decode_pb_package_id decoder decodes a package_id binary value from decoder

Sourceval decode_pb_package_version : Pbrt.Decoder.t -> package_version

decode_pb_package_version decoder decodes a package_version binary value from decoder

Sourceval decode_pb_position : Pbrt.Decoder.t -> position

decode_pb_position decoder decodes a position binary value from decoder

Sourceval decode_pb_range : Pbrt.Decoder.t -> range

decode_pb_range decoder decodes a range binary value from decoder

Sourceval decode_pb_key : Pbrt.Decoder.t -> key

decode_pb_key decoder decodes a key binary value from decoder

Sourceval decode_pb_dependency : Pbrt.Decoder.t -> dependency

decode_pb_dependency decoder decodes a dependency binary value from decoder

Sourceval decode_pb_value_type : Pbrt.Decoder.t -> value_type

decode_pb_value_type decoder decodes a value_type binary value from decoder

Sourceval decode_pb_checksum : Pbrt.Decoder.t -> checksum

decode_pb_checksum decoder decodes a checksum binary value from decoder

Sourceval decode_pb_value : Pbrt.Decoder.t -> value

decode_pb_value decoder decodes a value binary value from decoder

Sourceval decode_pb_constructive_trace : Pbrt.Decoder.t -> constructive_trace

decode_pb_constructive_trace decoder decodes a constructive_trace binary value from decoder